// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


/*
   El objetivo de ste programa es generar errores en el cliente que se conecte a l.

   Como no atiendo ningn tipo de.trfico llegar un momento en el que su cola de entrada
   alcanzar el lmite y el cliente obtendr un error al intentar enviar el mensaje.

   Se trata de verificar que los servicios de reparto son capaces de asumir ciertos
   tipos de error.
*/
#include <iostream>
#include <signal.h>

#include <anna/core/core.hpp>
#include <anna/comm/comm.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

using namespace std;

class MyCommunicator : public Communicator {
public:
   MyCommunicator () {;}
      
private:
   void eventReceiveMessage (comm::ClientSocket&, const Message&) throw (RuntimeException);
};

class Blocker : public comm::Application {
public:
   Blocker ();
      
private:
   MyCommunicator a_communicator;
   comm::ServerSocket* a_blockerSocket;

   void initialize () throw (RuntimeException);
   void run () throw (RuntimeException);
};

using namespace std;
using namespace anna::comm;

int main (int argc, const char** argv)
{
   CommandLine& commandLine (CommandLine::instantiate ());
   Blocker app;

   srand (time (NULL));

   sigignore (SIGABRT);
   
   try {
      commandLine.initialize (argv, argc);
      commandLine.verify ();

      Logger::setLevel (Logger::Debug); 
      Logger::initialize ("Blocker", new TraceWriter ("file.trace",4096000));
 
      app.start ();
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }
   
   return 0;
}

Blocker::Blocker () : 
   Application ("Blocker", "Bloqueador de comunicaciones", "1.0") 
{
   CommandLine& commandLine (CommandLine::instantiate ());
      
   commandLine.add ("p", CommandLine::Argument::Mandatory, "Puerto en el que atender peticiones");
   commandLine.add ("a", CommandLine::Argument::Mandatory, "Direccin IP en la que atender");
   commandLine.add ("trace", CommandLine::Argument::Optional, "Nivel de trazas (debug,warning, error,...)");
}

//-----------------------------------------------------------------------------------------
// Inicializa el servidor de sockets.
//-----------------------------------------------------------------------------------------
void Blocker::initialize () 
   throw (RuntimeException)
{
   CommandLine& cl (CommandLine::instantiate ());

   int port = cl.getIntegerValue ("p");
   const comm::Device* device = Network::instantiate ().find (Device::asAddress (cl.getValue ("a")));

   a_blockerSocket = new ServerSocket (INetAddress (device, port), false);
   a_blockerSocket->setCategory (777);
}

//-----------------------------------------------------------------------------------------
// Atiende las peticiones.
// Cuando hay un nuevo mensaje invocar�a Communicator::eventReceiveMessage
//-----------------------------------------------------------------------------------------
void Blocker::run ()
   throw (RuntimeException)
{
   CommandLine& cl (CommandLine::instantiate ());

   a_communicator.attach (a_blockerSocket);

   if (cl.exists ("trace"))
      Logger::setLevel (Logger::asLevel (cl.getValue ("trace")));

   a_communicator.accept ();
}

//-----------------------------------------------------------------------------------------
// Cuando recibe el primer mensaje deja al sistema bloqueado esperando una tecla.
//-----------------------------------------------------------------------------------------
void MyCommunicator::eventReceiveMessage (ClientSocket& clientSocket, const Message& message)
   throw (RuntimeException)
{
   LOGMETHOD (TraceMethod tm ("MyCommunicator", "eventReceiveMessage", ANNA_FILE_LOCATION));
   static bool blocking = true;

   if (blocking == false) 
      return;

   char aux [1024];
   bool stop = false;

   app::Application& app = app::functions::getApp ();
   cout << "Bloqueando las comunicaciones ejecute (kill -9 " << app.getPid ()  << ") ... " << flush;
   while (stop == false) {
      if (fgets(aux, sizeof(aux), stdin) != NULL) {
         if (aux [0] == 'q' || aux [0] == 'Q')
            stop = true;
      }
      sleep (10);
   } 

   blocking = false;
}

