// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <iostream>
#include <functional>

#include <anna/core/core.hpp>
#include <anna/timex/MicroMeter.hpp>

using namespace std;

static const int N = 10000;

void print_const (const Recycler<int>& numbers)
{
   string msg;

   const int* c;
   for (Recycler <int>::const_iterator ii = numbers.begin (), maxii = numbers.end (); ii != maxii; ii ++) {
      c = Recycler <int>::data (ii);
      msg = anna::functions::asString ("Number 3 (p=0x%x): %d", c, *c);
      Logger::debug (msg, ANNA_FILE_LOCATION);
   }
}

void debug (const char* str, Recycler<int>::iterator ii)
   throw ()
{
   int* p = Recycler<int>::data (ii);

   string msg (str);
   msg += " (p=";
   msg += functions::asHexString (anna_ptrnumber_cast (p));
   msg += functions::asText ("): ", *p);
   Logger::debug (msg, ANNA_FILE_LOCATION);
}

void run (const bool randomAccess, const int freeCounter) 
   throw (RuntimeException)
{
   Recycler <int> numbers (randomAccess);
   int* p;
   int i;
   string msg;

   // Crear N elementos
   for (i = 0; i < N; i ++) {
      p = numbers.create ();
      *p = i;
   }

   // Los recorre
   for (Recycler <int>::iterator ii = numbers.begin (), maxii = numbers.end (); ii != maxii; ii ++) 
      debug ("Number 1", ii);

   // Calcule cuantos objetos tiene que liberar
   int number;
   
   for (int nn = 0; nn < freeCounter; nn ++) {
      number = rand () % (N * 2);

      for (Recycler <int>::iterator ii = numbers.begin (), maxii = numbers.end (); ii != maxii; ii ++) {
         p = Recycler <int>::data (ii);

         if (*p == number) {
            debug ("Release | ", ii);
            numbers.release (p);
            break;
         }
      }
   }
      
   for (i = 0; i < N; i ++) {
      p = numbers.create ();
      *p = (1 + i) * 1000;
   }
   for (Recycler <int>::iterator ii = numbers.begin (), maxii = numbers.end (); ii != maxii; ii ++) 
      debug ("Number 2", ii);

   numbers.clear ();
   for (i = 0; i < N; i ++) {
      p = numbers.create ();
      *p = (10 + i) * 2000;
   }

   print_const (numbers);
}

void recycling (const bool randomAccess, const int freeCounter)
   throw (RuntimeException)
{
   Recycler <int> numbers (randomAccess);
   int** pp;
   int* p;
   int i;
 
   pp = new int* [N];

   // Creating N items 
   for (i = 0; i < N; i ++) {
      p = pp [i] = numbers.create ();
      *p = i;
   }

   // How many objects must be released? 
   int number;

   for (int nn = 0; nn < freeCounter; nn ++) {
      number = rand () % N;
      p = pp [number];
      numbers.release (p);
   }
}

void reuse (const int freeCounter)
   throw (RuntimeException)
{
   int** pp;
   int* p;
   int i;
 
   pp = new int* [N];

   // Creating N items 
   for (i = 0; i < N; i ++) {
      p = pp [i] = new int;
      *p = i;
   }

   // How many objects must be released? 
   int number;

   for (int nn = 0; nn < freeCounter; nn ++) {
      number = rand () % N;
      p = pp [number];
      delete p;
      pp [number] = NULL;
   }
}
int main (int argc, const char** argv)
{
   anna::timex::MicroMeter meter;

   try {
      Logger::setLevel (Logger::Debug); 
      Logger::initialize ("recycler", new TraceWriter ("file.trace", 2048000));
 
      const int freeCounter = rand () % N;
   
      Logger::debug (functions::asText ("Nodes to delete: ", freeCounter), ANNA_FILE_LOCATION);

      cout << "Nodes to delete: " << freeCounter << endl << endl;

      run (false, freeCounter);
      cout << "run (false, freeCounter): " << meter.getDelay () << " us" << endl;

      meter.setControlPoint ();
      run (true, freeCounter);
      cout <<  "run (true, freeCounter): " << meter.getDelay () << " us" << endl;

      meter.setControlPoint ();
      recycling (false, freeCounter);
      cout << "recycling(false): " << meter.getDelay () << " us" << endl;

      meter.setControlPoint ();
      recycling (true, freeCounter);
      cout <<  "recycling(true): " << meter.getDelay () << " us" << endl;

      meter.setControlPoint ();
      reuse (freeCounter);
      cout <<  "reuse(): " << meter.getDelay () << " us" << endl;
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }

   Recycler <string> rstr;
   meter.setControlPoint ();
   
   
   return 0;
}

