// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_RuntimeException_hpp
#define anna_core_RuntimeException_hpp

#include <errno.h>
#include <string.h>

#include <anna/core/Exception.hpp>

namespace anna {

/**
   Excepcion lanzada durante la ejecucion de un caso de prueba al comprobar
   que alguna situacion es anomala.
*/
class RuntimeException : public Exception {
public:
  /**
    Constructor.
  *
    @param text Texto con la explicacion de la excepcion.
    @param fromFile Fichero en el que se provoco la situacion de error.
    @param fromLine Linea del fichero en la que se provoco la situacion de error.
  */
  RuntimeException(const char* text, const char* fromFile, const int fromLine) :
    Exception(text, "RuntimeException", fromFile, fromLine),
    a_errno(-1) {}

  /**
    Constructor.
  *
    @param text Texto con la explicacion de la excepcion.
    @param fromFile Fichero en el que se provoco la situacion de error.
    @param fromLine Linea del fichero en la que se provoco la situacion de error.
  */
  explicit RuntimeException(const std::string& text, const char* fromFile, const int fromLine) :
    Exception(text.c_str(), "RuntimeException", fromFile, fromLine),
    a_errno(-1) {}

  /**
    Constructor.
  *
    @param fileName Nombre del fichero sobre el que actuabamos
    \param xerrno Numero de error (errno) generado por la ultima operacion de IO o SO.
    @param fromFile Fichero en el que se provoco la situacion de error.
    @param fromLine Linea del fichero en la que se provoco la situacion de error.
  */
  explicit RuntimeException(const std::string& fileName, const int xerrno, const char* fromFile, const int fromLine) :
    Exception(std::string(fileName + ": "  + strerror(xerrno)).c_str(), "RuntimeException", fromFile, fromLine),
    a_errno(xerrno) {}

  /**
    Constructor Copia.
    @param other Instancia de la que obtener la informacion.
  */
  RuntimeException(const RuntimeException& other) : Exception(other),
    a_errno(other.a_errno)
  {;}

  /**
    Constructor Copia.
    @param other Instancia de la que obtener la informacion.
  */
  RuntimeException(const Exception& other) : Exception(other),
    a_errno(-1)
  {;}

  /**
    @return El numero de error con el que se inicio la excepcion.
  */
  int getErrorNumber() const throw() { return a_errno; }

private:
  const int a_errno;
};

} //namespace anna

#endif

