// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_http_MessageFactory_hpp
#define anna_http_MessageFactory_hpp

#include <anna/core/Singleton.hpp>
#include <anna/core/util/Recycler.hpp>

#include <anna/http/Request.hpp>
#include <anna/http/Response.hpp>

namespace anna {

namespace http {

/**
   Factoria de mensajes HTTP.
*/
class MessageFactory : public Singleton <MessageFactory> {
public:
  /**
     Crea un mensaje HTTP del tipo correspondiente al recibido como parametro.
     \param type Tipo de mensaje HTTP que deseamos crear.
     \return Una nueva instancia de mensaje HTTP.
     \warning Cada una de las instancias recibidas con este metodo debe ser liberada con #release.
  */
  Message* create(const Message::Type::_v type) throw(RuntimeException);

  /**
     Libera los recursos del mensaje HTTP recibido como parametro.
     \param message Mensaje HTTP a liberar.
     \warning Si el mensaje recibido no fue obtenido mediante #create los resultado obtenidos no
     estan documentados.
  */
  void release(Message* message) throw();

private:
  Recycler <Request> a_requests;
  Recycler <Response> a_responses;

  MessageFactory() {;}

  friend class Singleton<MessageFactory>;
};


}
}

#endif
