// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_io_AbstractReader_hpp
#define anna_io_AbstractReader_hpp

#include <stdio.h>
#include <string>

#include <anna/core/RuntimeException.hpp>

namespace anna {

namespace io {

/**
   Lector de ficheros de texto.
*/
class AbstractReader {
public:
  /**
     Destructor.
  */
  virtual ~AbstractReader();

  /**
     Devuelve la estructura de bajo nivel con la que leemos el archivo.
     \return La estructura de bajo nivel con la que leemos el archivo.
  */
  FILE* getFile() throw() { return a_file; }

  /**
     Devuelve \em true si el fichero indicado en el constructor puede ser tratado
     \em false en otro caso.
     \return \em true si el fichero indicado en el constructor puede ser tratado
     \em false en otro caso.
  */
  bool isok() const throw() { return a_file != NULL; }

  /**
     Prepara esta instancia para trabajar con un nuevo archivo.
     \param filename Nombre del archivo con el que trabajar.
  */
  void open(const std::string& filename) throw(anna::RuntimeException);

  /**
     Libera los recursos del fichero.
  */
  void close() throw();

protected:
  FILE* a_file;

  /**
     Constructor.
     \param maxLength Longitud maxima de linea del fichero a tratar.
     \warning Con esta clase solo podemos tratar archivos de texto. Si utilizamos este constructor
     debemos invocar al mtodo open antes de usar cualquier otro mtodo de la clase.
  */
  AbstractReader();

  /**
     Constructor.
     \param filename Ruta completa del fichero a leer.
     \warning Con esta clase solo podemos tratar archivos de texto.
  */
  explicit AbstractReader(const char* filename);

  /**
     Constructor.
     \param filename Ruta completa del fichero a leer.
     \warning Con esta clase solo podemos tratar archivos de texto.
  */
  explicit AbstractReader(const std::string& filename);

  /*
   * Verifica que el fichero fu abierto correctamente.
   * \warning Exclusivamente de uso interno.
   */
  void verify() throw(RuntimeException);

private:
  const std::string a_filename;
  RuntimeException* a_ex;
};

}
}
#endif
