// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_xml_Compiler_hpp
#define anna_xml_Compiler_hpp

#include <anna/config/defines.hpp>
#include <anna/core/RuntimeException.hpp>
#include <anna/core/mt/Mutex.hpp>

// Three-space tabulator for xml string presentation (node indent)
#define ANNA_XML_COMPILER_TAB  "   "

namespace anna {

namespace xml {

class Node;
class Document;
class Attribute;

/**
   Genera el documento XML a correspondiente a un nodo anna::xml::Node.
*/
class Compiler : public Mutex {
public:
  /**
     Modo de generacion del documento XML.
  */
  struct Mode {
    enum _v {
      Visual = 0, /**< Separa cada uno de los componentes con espacios y retornos de carro */
      Compact = 1, /**< Los componentes no tienen separacion entre si. Se puede usar para optimizar envios de mensajes */
      ShowNullAttribute = 2, /**< Visualiza el valor nulo de un atributo como ="" */
      NoNamespaces = 4 /**<Obtiene el documento XML sin ofrecer informacin sobre los namespaces */
    };
  };

  /**
     Constructor.
  */
  Compiler();

  /**
     Obtiene el documento XML correspondiente al nodo recibido.
     \param node Nodo XML del que queremos obtener el documento XML origen.
     \param flag Combinacion de valores que configura la forma de crear el documento.
     \return Una cadena con el documento XML correspondiente al nodo recibido.
  */
  const char* apply(const Node* node, const int flag = Mode::Visual) throw(RuntimeException);

  /**
     Obtiene el documento XML correspondiente al nodo recibido.
     \param document Documento del que obtener los ajustes de configuracin del documento XML.
     \param node Nodo XML del que queremos obtener el documento XML origen.
     \param flag Combinacion de valores que configura la forma de crear el documento.
     \return Una cadena con el documento XML correspondiente al nodo recibido.
  */
  const char* apply(const Document& document, const Node* node, const int flag = Mode::Visual) throw(RuntimeException);

private:
  class Result {
  public:
    Result() {
      a_buffer = new char [a_maxSize = 8192];
      a_buffer [a_size = 0] = 0;
    }
    ~Result() { delete [] a_buffer; }

    void clear() throw() { a_buffer [a_size = 0] = 0; }

    void operator += (const char c)
    throw(RuntimeException) {
      extend(sizeof(c) * 2);
      a_buffer [a_size ++] = c;
      a_buffer [a_size] = 0;
    }

    void operator += (const char* str)
    throw(RuntimeException) {
      const int len = anna_strlen(str);
      extend(len + 1);
      anna_strcpy(a_buffer + a_size, str);
      a_size += len;
    }

    void operator += (const std::string& str) throw(RuntimeException) {(*this) += str.c_str(); }

    const char* c_str() const throw() { return a_buffer; }

  private:
    char* a_buffer;
    int a_size;
    int a_maxSize;

    void extend(const int size) throw(RuntimeException);
  };

  Result a_result;

  Compiler(const Compiler&);

  static void apply(const Node*, Result&, const int level, const int flags) throw(RuntimeException);
  static void apply(const Node*, Result&, const int flags) throw(RuntimeException);
  static void open(const Node*, Result&, const int level, const bool quickClose, const bool newline, const int flags)
  throw(RuntimeException);
  static void writeFullName(const Node*, Result&, const int flags) throw(RuntimeException);
  static void writeFullName(const Attribute*, Result&, const int flags) throw(RuntimeException);
  static void close(const Node*, Result&, const int level, const int flags) throw(RuntimeException);
};

}
}

#endif
