// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <pthread.h>
#include <errno.h>

#include <anna/core/mt/Mutex.hpp>
#include <anna/config/defines.hpp>

using namespace anna;

Mutex::Mutex(const Mutex::Mode::_v mode) {
#ifdef _MT

  if(mode == Mode::Recursive) {
    pthread_mutexattr_t mattr;
    pthread_mutexattr_init(&mattr);
    pthread_mutexattr_settype(&mattr, PTHREAD_MUTEX_RECURSIVE_NP);  // PTHREAD_MUTEX_RECURSIVE in Solaris
    pthread_mutex_init(&a_id, &mattr);
    pthread_mutexattr_destroy(&mattr);
  } else
    pthread_mutex_init(&a_id, NULL);

#endif
}

Mutex::~Mutex() {
#ifdef _MT
  pthread_mutex_destroy(&a_id);
#endif
}

void Mutex::lock()
throw(RuntimeException) {
#ifdef _MT
  int errorCode;

  if((errorCode = pthread_mutex_lock(&a_id)) != 0)
    throw RuntimeException(std::string("pthread_mutex_lock"), errorCode, ANNA_FILE_LOCATION);

#endif
}

void Mutex::unlock()
throw() {
#ifdef _MT
  int errorCode;

  try {
    if((errorCode = pthread_mutex_unlock(&a_id)) != 0)
      throw RuntimeException(std::string("pthread_mutex_unlock"), errorCode, ANNA_FILE_LOCATION);
  } catch(Exception& ex) {
    ex.trace();
  }

#endif
}

bool Mutex::trylock()
throw(RuntimeException) {
#ifdef _MT
  int errorCode;

  try {
    if((errorCode = pthread_mutex_trylock(&a_id)) != 0) {
      if(errorCode == EBUSY)
        return false;

      throw RuntimeException(std::string("pthread_mutex_trylock"), errorCode, ANNA_FILE_LOCATION);
    }
  } catch(Exception& ex) {
    ex.trace();
  }

#endif
  return true;
}


