// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <stdlib.h>

#include <anna/core/functions.hpp>

#include <anna/core/util/Encoder.hpp>


using namespace std;
using namespace anna;

void Encoder::initialize()
throw() {
  srand(functions::second());
}

const EncodedData& Encoder::encode(const DataBlock& data)
throw(RuntimeException) {
  DES_cblock key [3];
  DES_key_schedule* skey [3];

  for(int i = 0; i < 3; i ++) {
    DES_random_key(&key[i]);
    skey [i] = &a_data.a_skey [i];

    if(DES_set_key_checked(&key[i], skey [i]) < 0) {
      string msg("anna::Encoder::encode | DataBlock: ");
      msg += functions::asString(data);
      msg += " | Error during key generation";
      throw RuntimeException(msg, ANNA_FILE_LOCATION);
    }
  }

  errno = 0;
  anna_memset(&a_data.a_iv, 0, sizeof(DES_cblock));
  const DataBlock& output = setDataBlock(data);
  unsigned char* aux = (unsigned char*) output.getData();
  DES_ede3_cbc_encrypt(aux, aux, output.getSize(), skey[0], skey[1], skey[2], &a_data.a_iv, DES_ENCRYPT);

  if(errno != 0) {
    string msg("anna::Encoder::encode | DataBlock: ");
    msg += functions::asString(data);
    msg += " | Encoding error";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  return a_data;
}

//-------------------------------------------------------------------------------------------------
// (1) Elimina los 8 primeros bytes que tuvimos que poner de relleno en la codificacion.
//-------------------------------------------------------------------------------------------------
const DataBlock& Encoder::decode(const EncodedData& data)
throw(RuntimeException) {
  if(data.a_realSize <= 0) {
    string msg("anna::Encoder::encode | Cannot decode an uninitialized data");
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  unsigned char* text = (unsigned char*) data.a_value.getData();
  DES_key_schedule* skey [3];
  DES_cblock* iv;
  skey [0] = const_cast <DES_key_schedule*>(&data.a_skey [0]);
  skey [1] = const_cast <DES_key_schedule*>(&data.a_skey [1]);
  skey [2] = const_cast <DES_key_schedule*>(&data.a_skey [2]);
  iv = const_cast <DES_cblock*>(&data.a_iv);
  errno = 0;
  DES_ede3_cbc_encrypt(text, text, data.a_value.getSize(), skey[0], skey[1], skey[2], iv, DES_DECRYPT);

  if(errno != 0) {
    string msg("anna::Encoder::encode | Decoding error | Block: ");
    msg += functions::asString(data.a_value);
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  const_cast <EncodedData&>(data).a_value.remove(sizeof(DES_cblock));        // (1)

  const_cast <EncodedData&>(data).a_value.allocate(data.a_realSize);

  return data.a_value;
}

//-------------------------------------------------------------------------------------------------
// (1) Rellena los 8 primeros bytes que no sabemos interpretar correctamente, lo ponemos para
// poder ignorarlos al decodificar. Ver
// 'const_cast <EncodedData&> (data).a_value.remove (sizeof (DES_cblock));'
//-------------------------------------------------------------------------------------------------
DataBlock& Encoder::setDataBlock(const DataBlock& other)
throw(RuntimeException) {
  DataBlock& value = a_data.a_value;
  int r;
  value.clear();

  for(r = 0; r < sizeof(DES_cblock); r ++)                                      // (1)
    value += char(0xaa);

  value += other;
  r = (a_data.a_realSize = other.getSize()) % 8;

  if(r > 0) {
    for(r = 8 - r; r > 0; r --)
      value += char(0xee);
  }

  return value;
}



