// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/diameter.comm/Timer.hpp>
#include <anna/diameter.comm/Response.hpp>
#include <anna/diameter.comm/Session.hpp>
#include <anna/diameter.comm/LocalServer.hpp>

#include <anna/core/tracing/Logger.hpp>


using namespace std;
using namespace anna;

void diameter::comm::Timer::expire(anna::timex::Engine*)
throw(anna::RuntimeException) {
  Response* response = NULL;
  Session* session = NULL;
  LocalServer* localServer = NULL;

  switch(getType()) {
  case Type::ResponseExpiration:
    response = getResponse();
    response->setResultCode(Response::ResultCode::Timeout);
    session = response->getSession();
    LOGDEBUG(
      string msg("anna::diameter::comm::Timer::expire | Response: ");
      msg += response->asString();
      anna::Logger::debug(msg, ANNA_FILE_LOCATION);
    );
    // action:
    session->expireResponse(response);
    break;
  case Type::SessionUnbind:
    session = getSession();

    if(session) {
      LOGDEBUG(
        string msg("anna::diameter::comm::Timer::expire | Session (ending): ");
        msg += session->asString();
        anna::Logger::debug(msg, ANNA_FILE_LOCATION);
      );
      // action:
      //session->setOnDisconnect(Session::OnDisconnect::IgnorePendings);
      session->unbind(true /* immediate */);
    } else {
      // La client-session desaparecio (hice un closeSession)
      LOGDEBUG(anna::Logger::debug("anna::diameter::comm::Timer::expire | Session missing", ANNA_FILE_LOCATION));
    }

    break;
  case Type::SessionRecover:
    session = getSession();

    if(session) {
      LOGDEBUG(
        string msg("anna::diameter::comm::Timer::expire | Session (recovering): ");
        msg += session->asString();
        anna::Logger::debug(msg, ANNA_FILE_LOCATION);
      );
      // action:
      session->setState(Session::State::Bound);
    } else {
      // La client-session desaparecio (hice un closeSession)
      LOGDEBUG(anna::Logger::debug("anna::diameter::comm::Timer::expire | Session missing", ANNA_FILE_LOCATION));
    }

    break;
  case Type::LocalServerAttach:
    localServer = getLocalServer();

    if(localServer) {
      LOGDEBUG(
        string msg("anna::diameter::comm::Timer::expire | LocalServer (attach retry)");
        //msg += localServer->asString();
        anna::Logger::debug(msg, ANNA_FILE_LOCATION);
      );
      // action:
      localServer->attach();
    } else {
      // LocalServer desaparecio (hice un closeLocalServer)
      LOGDEBUG(anna::Logger::debug("anna::diameter::comm::Timer::expire | LocalServer missing", ANNA_FILE_LOCATION));
    }

    break;
  }
}

const char* diameter::comm::Timer::asText(const Type::_v type)
throw() {
  static const char* text [] = { "ResponseExpiration", "SessionUnbind", "SessionRecover", "LocalServerAttach" };
  return text [type];
}

string diameter::comm::Timer::asString() const
throw() {
  string result("anna::diameter::comm::Timer { ");
  result += anna::timex::Transaction::asString();
  result += " Type: ";
  result += asText(a_type);

  switch(getType()) {
  case Type::ResponseExpiration: {
    const Response* response = getResponse();

    if(response != NULL) {
      result += " | ";
      result += response->asString();
    } else
      result += " | Response: <null>";

    break;
  }
  case Type::SessionUnbind:
  case Type::SessionRecover: {
    const Session *session = getSession();

    if(session != NULL) {
      result += " | ";
      result += session->asString();
    } else
      result += " | Session: <null>"; // puede ocurrir ??

    break;
  }
  case Type::LocalServerAttach: {
    const LocalServer *localServer = getLocalServer();

    if(localServer != NULL) {
      result += " | ";
      result += localServer->asString();
    } else
      result += " | LocalServer: <null>"; // puede ocurrir ??

    break;
  }
  }

  return result += " }";
}

