// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //

// Standard
#include <string>
#include <fstream>

// Project
#include <anna/diameter/codec/Message.hpp>
#include <anna/diameter/codec/Engine.hpp>
#include <anna/diameter/codec/EngineManager.hpp>

// Process
#include <anna/diameter/codec/MessagesDeque.hpp>


using namespace anna::diameter::codec;


void MessagesDeque::clear() throw () {
  try {
    anna::diameter::codec::EngineManager &em = anna::diameter::codec::EngineManager::instantiate();
    anna::diameter::codec::Engine *engine;

    for (messages_const_iterator it = a_deques.begin(); it != a_deques.end(); it++) {
      anna::diameter::codec::Message *message = *(it->second->begin());
      engine = em.getCodecEngine(message->getApplicationId());
      if (engine) {
        engine->releaseMessage(message);
        delete (it->second);
      }
      else {
        LOGWARNING(anna::Logger::warning("Cannot release a message for which i don't know the codec engine (check the registered stack id regarding the message application id) !", ANNA_FILE_LOCATION));
      }
    }
    a_deques.clear();
  }
  catch (anna::RuntimeException &ex) {
    ex.trace();
  }
}

void MessagesDeque::dump(const char *filenamePrefix) throw () {
  std::string outfilename, xmlString;
  for (messages_const_iterator it = a_deques.begin();
      it != a_deques.end(); it++) {
    int sequence = 1;
    for (codec_messages_deque_const_iterator itm = it->second->begin();
        itm != it->second->end(); itm++) {
      // message.<code>.<sequence>
      outfilename = filenamePrefix;
      outfilename += ".";
      outfilename += anna::functions::asString(it->first);
      outfilename += ".";
      outfilename += anna::functions::asString(sequence++);
      outfilename += ".xml";
      std::ofstream outfile(outfilename.c_str(), std::ifstream::out);
      xmlString = (*itm)->asXMLString();
      outfile.write(xmlString.c_str(), xmlString.size());
      outfile.close();
    }
  }
}

void MessagesDeque::addMessage(int code, anna::diameter::codec::Message *message) throw () {
  if (!message)
    return; // just in case

  messages_const_iterator it = a_deques.find(code);
  if (it != a_deques.end()) {
    it->second->push_back(message);
  } else {
    codec_messages_deque *deque = new codec_messages_deque;
    a_deques[code] = deque;
    deque->push_back(message);
  }
}

anna::diameter::codec::Message* MessagesDeque::getMessage(int code) const throw () { //get the front message (begin()), returns NULL if deque is empty
  anna::diameter::codec::Message *result = NULL;
  messages_const_iterator it = a_deques.find(code);
  if (it != a_deques.end()) {
    if (!it->second->empty())
      result = *(it->second->begin());
  }
  return result;
}

void MessagesDeque::nextMessage(int code) throw () { //pops the deque and release the message (when deque is not empty: deque::empty)
  anna::diameter::codec::Engine *engine;

  try {
    messages_const_iterator it = a_deques.find(code);
    if (it != a_deques.end()) {
      if (!it->second->empty()) {
        anna::diameter::codec::Message *message = *(it->second->begin());
        if (a_rotate) {
          addMessage(code, message);
        } else {
          engine = anna::diameter::codec::EngineManager::instantiate().getCodecEngine(message->getApplicationId());
          if (engine) {
            engine->releaseMessage(message);
          }
          else {
            LOGWARNING(anna::Logger::warning("Cannot release a message for which I don't know the codec engine (check the registered stack id regarding the message application id) !", ANNA_FILE_LOCATION));
            return;
          }
        }
        it->second->pop_front();
      }
    }
  }
  catch (anna::RuntimeException &ex) {
    ex.trace();
  }
}

std::string MessagesDeque::asString(const char *queueName) const throw () {
  std::string result = "";
  std::string aux = "FIFO QUEUE '";
  aux += queueName;
  aux += "', Rotation ";
  aux += a_rotate ? "enabled" : "disabled";
  result += anna::functions::highlightJustify(aux);
  if (a_deques.size() != 0) {
    for (messages_const_iterator it = a_deques.begin();
        it != a_deques.end(); it++) {
      if (it->second->size() != 0) {
        aux = "Message code ";
        aux += anna::functions::asString(it->first);
        result += anna::functions::highlightJustify(aux,
            anna::functions::TextHighlightMode::OverAndUnderline,
            anna::functions::TextJustifyMode::Left, '-');
        for (codec_messages_deque_const_iterator itm = it->second->begin();
            itm != it->second->end(); itm++) {
          result += (*itm)->asXMLString();
          result += "\n";
        }
        result += "\n";
      }
    }
  } else {
    result = "No ocurrences found\n\n";
  }
  return result;
}
